/*
*Unreal World
* Deklaracja klas dźwiękowych
*Designed by Arkadiusz kozioł at 22:4:57 12 mar 2016
*/

#include <audio.h>
/* klasa podstawowa do zarządzania dźwiękiem w menu*/
Audio::Audio()
{

	if (FMOD::System_Create(&sysfmod) != FMOD_OK)
	{
		logmsg("Audio: Failed to create sysfmod");
	}
	if (sysfmod->init(1024, FMOD_INIT_NORMAL, 0) != FMOD_OK)
	{
		logmsg("Audio: Failed to initialize");
	}
	if (sysfmod->set3DSettings(1.0, dfactor, 1.0f) != FMOD_OK)
	{
		logmsg("Audio: An error occurred while setting 3d settings");
	}
		loadSounds(SND_ALL);
}
Audio::Audio(const Audio &)
{
}
Audio & Audio::init()
{
static Audio audio;
return audio;
}
bool Audio::loadMusic()
{
logmsg("Audio: loading UI music");
for(int k = 0; k < M_COUNT; k++)
{
	if (!music[k].openFromFile("audio/" + name2[k]))
	{
		logmsg("Audio error: Failed to open audio/m/%s. No such file or directory", (name2[k]).c_str());
		return false;
	}
}
logmsg("Audio: success");
return true;
}
void Audio::loading()
{
loadMusic();
playMusic(LOADING);
setMusicLoop(LOADING, true);
}
bool Audio::loadSounds(SNDLoad sndt)
{
if(loaded == false)
{
loading();
		logmsg("Audio: Loading sounds");
if(sndt == SND_ALL)
{
		for (int i = 0; i < COUNT; i++)
	{
		iCount = i;
if (sysfmod->createSound(("audio/" + name[i]).c_str(), FMOD_3D, 0, &sound[i]) != FMOD_OK)
		{
			logmsg("Audio error: Failed to load audio/%s. No such file or directory", (name[i]).c_str());
			return false;
		}
	}
iIndex = 0;
}
		else if(sndt == SND_GAME)
{
for (int i = 10; i < COUNT; i++)
	{
iCount = i;
		if (sysfmod->createSound(("audio/" + name[i]).c_str(), FMOD_3D, 0, &sound[i]) != FMOD_OK)
		{
			logmsg("Audio error: Failed to load audio/%s. No such file or directory", (name[i]).c_str());
			return false;
		}
	}
		
iIndex = 10;
}
else if(sndt == SND_MENU)
{
iIndex = 0;
for (int i = 0; i < 10; i++)
	{
iCount = i;
		if (sysfmod->createSound(("audio/" + name[i]).c_str(), FMOD_3D, 0, &sound[i]) != FMOD_OK)
		{
			logmsg("Audio error: Failed to load audio/%s. No such file or directory", (name[i]).c_str());
			return false;
		}
	}
		
}
loaded = true;
stopMusic(LOADING);
	logmsg("Audio: success");
logmsg("Audio: Index %i", iIndex);	
return true;
}
else
{
logmsg("Audio: Sounds loaded");
return true;
}
}
void Audio::playSound(sound_type type)
{
	if (sysfmod->playSound(sound[type], 0, true, &chan[type]) != FMOD_OK)
	{
		logmsg("Audio: An error occurred while playing a sound");
	}
	chan[type]->setPaused(false);
chan[type]->set3DDopplerLevel(1.0);
}
void Audio::play3DSound(sound_type type, float x, float y, float z)
{
	FMOD_VECTOR pos = { x * dfactor, y * dfactor, z * dfactor };
	FMOD_VECTOR vel = { 0.0f, 0.0f, 0.0f };
	if (sysfmod->playSound(sound[type], 0, true, &chan[type]) != FMOD_OK)
	{
		logmsg("Audio: An error occurred while playing a sound");
	}
	chan[type]->set3DAttributes(&pos, &vel);
	chan[type]->setPaused(false);
chan[type]->set3DDopplerLevel(1.0);}
Audio::~Audio()
{
if(rel == false)
{
	for (int i = iIndex; i < iCount; i++)
	{
		sound[i]->release();
	}
rel = true;
}
	sysfmod->close();
	sysfmod->release();
	logmsg("Audio: success");
}
bool Audio::unloadSounds()
{
	for (int i = iIndex; i <= iCount; i++)
	{
		if(sound[i]->release() == FMOD_OK)
{
logmsg("Audio: Fail at %i", i);
}
	}
	sysfmod->close();
	sysfmod->release();
	logmsg("Audio: success");
	return true;
}
bool Audio::unloadSound(sound_type t)
{
if(sound[t]->release() == FMOD_OK)
{
return true;
}
else
{
return false;
}
}
bool Audio::loadSound(sound_type t)
{
if (sysfmod->createSound(("audio/" + name[t]).c_str(), FMOD_3D, 0, &sound[t]) != FMOD_OK)
{
return false;
}
return true;
}
void Audio::playMusic(m_type m)
{
	music[m].play();
}
void Audio::stopMusic(m_type m)
{
	music[m].stop();
}
void Audio::setSoundPitch(sound_type type, float p)
{
	if (chan[type]->setPitch(p) != FMOD_OK)
	{
		logmsg("Audio: An error occurred while setting sound pitch");
	}
}
void Audio::setSoundVolume(sound_type type, float v)
{
	if (chan[type]->setVolume(v) != FMOD_OK)
	{
		logmsg("Audio: An error occurred while setting sound volume");
	}
}
float Audio::getMusicVolume(m_type m)
{
return music[m].getVolume();
}
void Audio::setSoundPosition(sound_type type, float p1, float p2, float p3)
{
		FMOD_VECTOR poz = { p1 * dfactor, p2 * dfactor, p3 * dfactor};
		FMOD_VECTOR vel = { 0.0f, 0.0f, 0.0f };
		if (chan[type]->set3DAttributes(&poz, &vel) != FMOD_OK)
		{
			logmsg("Audio: An error occurred while setting 3d parameters");
		}
	}
void Audio::setMusicVolume(m_type m, float v)
{
	music[m].setVolume(v);
}
void Audio::setSoundLoop(sound_type type, bool l)
{
	if (l == true)
	{
		chan[type]->setMode(FMOD_LOOP_NORMAL);
	}
	else
	{
		chan[type]->setMode(FMOD_LOOP_OFF);
	}
	}
void Audio::setMusicLoop(m_type m, bool l)
{
	music[m].setLoop(l);
}
void Audio::setMinMaxDistance(sound_type type, float dist, float dist2)
{
	if (sound[type]->set3DMinMaxDistance(dist * dfactor, dist2 * dfactor) != FMOD_OK)
	{
		logmsg("Audio: An error occurred while setting 3d distance");
	}
}
void Audio::stopSound(sound_type type)
{
	chan[type]->stop();
}
void Audio::update(float x, float y, float z)
{
lpos = {x, y, z};
	FMOD_VECTOR forward = { 0.0f, 0.0f, 1.0f };
	FMOD_VECTOR up = { 0.0f, 1.0f, 0.0f };
	FMOD_VECTOR vel = { 0.0f, 0.0f, 0.0f };
	sysfmod->set3DListenerAttributes(0, &lpos, &vel, &forward, &up);
	sysfmod->update();
}
